#ifdef __cplusplus
extern "C" {
#endif
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#ifdef __cplusplus
}
#endif

#include <sys/kernel.h>
#include <sys/types.h>
#include <sys/name.h>
#include <sys/proxy.h>
#include <errno.h>

MODULE = QNX		PACKAGE = QNX		

SV*
errno()
	CODE:
	RETVAL = newSViv( errno );
	OUTPUT:
	RETVAL

SV*
Send(sv_pid, smsg, rmsg, sv_size_send, sv_size_reply)
	SV* sv_pid
	SV* smsg
	SV* rmsg 
	SV* sv_size_send
	SV* sv_size_reply
	PREINIT:
	unsigned size_send = SvNV( sv_size_send );
	unsigned size_reply = SvNV( sv_size_reply );
	pid_t pid = SvIV( sv_pid );
	char* send_msg = SvPV( smsg, na );
	char* reply_msg = (char *) malloc( size_reply );
	int r;
	CODE:
	r = Send( pid , send_msg, reply_msg, size_send, size_reply);
	if( r != -1 )
		sv_setpvn( rmsg, reply_msg, size_reply );
	free( reply_msg );
	RETVAL = newSViv( r );
	OUTPUT:
	rmsg
	RETVAL

SV*
Receive(sv_pid, rvmsg, sv_size_receive)
	SV* sv_pid
	SV* rvmsg
	SV* sv_size_receive
	PREINIT:
	unsigned size_receive = SvNV( sv_size_receive );
	pid_t pid = SvIV( sv_pid );
	char* receive_msg = (char *) malloc( size_receive );
	int r;
	CODE:
	r = Receive( pid, receive_msg, size_receive );
	if( r != -1 )
		sv_setpvn( rvmsg, receive_msg, size_receive );
	free( receive_msg );
	RETVAL = newSViv( r );
	OUTPUT:
	rvmsg
	RETVAL

SV*
Reply(sv_pid, rpmsg, sv_size_reply)
	SV* sv_pid
	SV* rpmsg
	SV* sv_size_reply
	PREINIT:
	unsigned size_reply = SvNV( sv_size_reply );
	pid_t pid = SvIV( sv_pid );
	char* reply_msg = SvPV( rpmsg, na );
	CODE:
	RETVAL = newSViv( Reply( pid, reply_msg, size_reply ) );
	OUTPUT:
	RETVAL

SV*
name_attach(sv_nid, sv_name)
	SV* sv_nid
	SV* sv_name
	PREINIT:
	nid_t nid = SvIV( sv_nid );
	char* name = SvPV( sv_name, na );
	CODE:
	RETVAL = newSViv( qnx_name_attach( nid, name ) );
	OUTPUT:
	RETVAL

SV*
name_detach(sv_nid, sv_name_id)
	SV* sv_nid
	SV* sv_name_id
	PREINIT:
	nid_t nid = SvIV( sv_nid );
	int name_id = SvIV( sv_name_id );
	CODE:
	RETVAL = newSViv( qnx_name_detach( nid, name_id ) );
	OUTPUT:
	RETVAL

SV*
name_locate(sv_nid, sv_name, sv_size, sv_copies)
	SV* sv_nid
	SV* sv_name
	SV* sv_size
	SV* sv_copies
	PREINIT:
	nid_t nid = SvIV( sv_nid );
	char* name = SvPV( sv_name, na );
	unsigned size = SvNV( sv_size );
	unsigned copies = SvIV( sv_copies );
	unsigned* cptr = &copies;
	int r;
	CODE:
	r = qnx_name_locate( nid, name, size, cptr );
	if( r != -1 )	
		sv_setiv( sv_copies, copies );
	RETVAL = newSViv( r );
	OUTPUT:
	sv_copies
	RETVAL 

SV* 
proxy_attach(sv_pid, sv_data, sv_size, sv_priority)
	SV* sv_pid
	SV* sv_data
	SV* sv_size
	SV* sv_priority
	PREINIT:
	pid_t pid = SvIV( sv_pid );
	char* data = SvPV( sv_data, na );
	int size = SvIV( sv_size );
	int priority = SvIV( sv_priority );
	CODE:
	RETVAL = newSViv( qnx_proxy_attach( pid, data, size, priority ) );
	OUTPUT:
	RETVAL

SV*
proxy_detach(sv_pid)
	SV* sv_pid
	PREINIT:
	pid_t pid = SvIV( sv_pid );
	CODE:
	RETVAL = newSViv( qnx_proxy_detach( pid ) );
	OUTPUT:
	RETVAL

SV*
Trigger(sv_proxy)
	SV* sv_proxy
	PREINIT:
	pid_t proxy = SvIV( sv_proxy );
	CODE:
	RETVAL = newSViv( Trigger( proxy ) );
	OUTPUT:
	RETVAL

SV*
proxy_rem_attach(sv_nid, sv_proxy)
	SV* sv_nid
	SV* sv_proxy
	PREINIT:
	nid_t nid = SvIV( sv_nid );
	pid_t proxy = SvIV( sv_proxy );
	CODE:
	RETVAL = newSViv( qnx_proxy_rem_attach( nid, proxy ) );
	OUTPUT:
	RETVAL 

SV*
proxy_rem_detach(sv_nid, sv_vproxy)
	SV* sv_nid
	SV* sv_vproxy
	PREINIT:
	nid_t nid = SvIV( sv_nid );
	pid_t vproxy = SvIV( sv_vproxy );
	CODE:
	RETVAL = newSViv( qnx_proxy_rem_detach( nid, vproxy ) );
	OUTPUT:
	RETVAL
